#!/usr/bin/env python

"""**********************************************************************
  DLMS/COSEM components

  i-cube, 1376 Goumoens-la-Ville, Switzerland
          infos@icube.ch

 (C)Copyright 2004-2012
**********************************************************************

"""

__all__ = [ "HDLCProtocolState", "HDLCParameters" ]

__version__ = "2.1.3";

_CCFLAGS_ = [ "TRIAL" , ];

def get_version_str():
    module = "icube.ezhdlc, "
    trial = ""
    if "TRIAL" in _CCFLAGS_:
        trial += " - Trial";
    module  += __version__ + trial;
    return module;

class HDLCProtocolState(object):
    """HDLC protocol states."""

    NDM, NRM = range(2)
    
    @classmethod
    def str(cls, state):
        if state == HDLCProtocolState.NDM:
            return "Normal Disconnected Mode."
        elif state == HDLCProtocolState.NRM:
            return "Normal Response Mode."
        
class HDLCParameters(object):
    """ HDLC parameters."""
    
    def _get_ClientMACAddress(self):
        return self._ClientMACAddress
    def _set_ClientMACAddress(self, ClientMACAddress):
        self._ClientMACAddress = ClientMACAddress
    ClientMACAddress = property(_get_ClientMACAddress, _set_ClientMACAddress, 
                            "ushort Client MAC address.")
     
    def _get_ServerAddressSize(self):
        return self._ServerAddressSize
    def _set_ServerAddressSize(self, ServerAddressSize):
        self._ServerAddressSize = ServerAddressSize
    ServerAddressSize = property(_get_ServerAddressSize, _set_ServerAddressSize, 
                             "int Server address size.")

    def _get_ServerUpperMACAddressValue(self):
        return self._ServerUpperMACAddressValue
    def _set_ServerUpperMACAddressValue(self, ServerUpperMACAddressValue):
        self._ServerUpperMACAddressValue = ServerUpperMACAddressValue
    ServerUpperMACAddressValue = property(_get_ServerUpperMACAddressValue, _set_ServerUpperMACAddressValue, 
                                      "ushort ServerUpperMACAddressValue")

    def _get_ServerLowerMACAddressValue(self):
        return self._ServerLowerMACAddressValue
    def _set_ServerLowerMACAddressValue(self, ServerLowerMACAddressValue):
        self._ServerLowerMACAddressValue = ServerLowerMACAddressValue
    ServerLowerMACAddressValue = property(_get_ServerLowerMACAddressValue, _set_ServerLowerMACAddressValue, 
                                      "ushort Server Lower MAC address value.")

    def _get_SentStreamCallback(self):
        return self._SentStreamCallback
    def _set_SentStreamCallback(self, SentStreamCallback):
        self._SentStreamCallback = SentStreamCallback
    SentStreamCallback = property(_get_SentStreamCallback, _set_SentStreamCallback, 
                              "Callback to be called when a byte stream is sent to the device.")

    def _get_ReceivedStreamCallBack(self):
        return self._ReceivedStreamCallBack
    def _set_ReceivedStreamCallBack(self, ReceivedStreamCallBack):
        self._ReceivedStreamCallBack = ReceivedStreamCallBack
    ReceivedStreamCallBack = property(_get_ReceivedStreamCallBack, _set_ReceivedStreamCallBack, 
                                  "Callback to be called when a byte stream is received from the device.")

    def _get_StateChangeCallBack(self):
        return self._StateChangeCallBack
    def _set_StateChangeCallBack(self, StateChangeCallBack):
        self._StateChangeCallBack = StateChangeCallBack
    StateChangeCallBack = property(_get_StateChangeCallBack, _set_StateChangeCallBack, 
                               "Callback to be called when a State change occurs.")

if __name__ == "__main__":
    print "__version__ : %s" % __version__;
    print "_CCFLAGS_    : %s" % _CCFLAGS_;
    print "module      : %s" % get_version_str();
    
    