"""**********************************************************************
  DLMS/COSEM components

  i-cube, 1376 Goumoens-la-Ville, Switzerland
          infos@icube.ch

 (C)Copyright 2004-2012
**********************************************************************

"""

import types

from icube.xmlpdu.infra.xmlpduexception import XmlPduException
from icube.utils import Utils

class MemoryStream(object):
    """Implements a MemoryStream equivalent to C#'s MemoryStream."""
    INITIAL_CAPACITY = 20
    CAPACITY_STEP = INITIAL_CAPACITY
    
    def _get_bytes(self):
        return self._bytes[:self._length]
#    def set_bytes(self, bytes):
#        self._bytes = bytes
    _buffer = property(_get_bytes, "the stream bytes")
    buffer = property(_get_bytes, "the stream bytes")
            
    def __init__(self, bytes_=None):
        """Creates a MemoryStream and initializes it."""
        self.capacity = 0
        self._bytes = []
        self.position = 0
        self._length = 0
        if bytes_ == None:
            self.capacity = MemoryStream.INITIAL_CAPACITY
            self._bytes = [ 0 for x in range(MemoryStream.INITIAL_CAPACITY) ] # FIXME += [0,]*MemoryStream.INITIAL_CAPACITY
            self.position = 0
            self._length = 0
        else:
            self.capacity = len(bytes_)
            self._bytes = [ 0 for x in range(self.capacity) ]
            self.write(bytes_, 0, len(bytes_))
            self.position = 0

    def _get_capacity(self):
        return self._capacity
    def _set_capacity(self, pos):
        self._capacity = pos
    capacity = property(_get_capacity, _set_capacity, "Gets or sets the number of bytes allocated for this stream.")
    
    def _get_position(self):
        return self._position
    def _set_position(self, pos):
        self._position = pos
    position = property(_get_position, _set_position, "Gets or sets the current position within the stream.")
    
    def _get_length(self):
        return self._length
    def _set_length(self, value):
        self._length = value
    length = property(_get_length, _set_length, "Gets the length of the stream in bytes.")
    
    def erase(self, offset, count):
        """Erases 'count' bytes starting at 'offset'. Shuffling the data to fill 
        the erased data.
            @param offset: 
            @param count:  
        """
        for i in range(self._length - count - offset):
            to = offset + i
            _from = to + count
            self._bytes[to] = self._bytes[_from]
            done = True
        self._length -= count
        
    def write(self, bytes_, offset, count):
        """Writes a block of bytes_ to the current stream using data read from _buffer.
            bytes_    The _buffer to write data from.
            offset   The byte offset in _buffer at which to begin writing from.
            count    The maximum number of bytes_ to write.
        """
        for _i in range(offset, offset+count):
            self.write_byte(bytes_[_i])
    
    def write_byte(self, b):
        """Writes a byte to the current stream at the current position."""
        assert(isinstance(b, types.IntType))
        self.position += 1
        if self.position > self.capacity:
            self._bytes += [ 0 for x in range(MemoryStream.CAPACITY_STEP) ]
            self.capacity += MemoryStream.CAPACITY_STEP
        self._bytes[self.position-1] = b
        if self.position > self.length:
            self._length = self.position
            
    def read_byte(self):
        """Reads a byte from the current stream."""
        if self.position == self.length:
            return -1
        else:
            b = self._bytes[self.position]
            self.position += 1
            return b
        
    def __str__(self):
        _str = ""
        _str += "bytes: %s\n" % Utils.bytes_to_hex_string(self._bytes[:self.length])
        _str += "capacity: %d\n" % self.capacity
        _str += "position: %d\n" % self.position
        _str += "length: %d\n" % self.length
        return _str
    
class PduOutputStream(MemoryStream):
    """Implements a MemoryStream dedicated to pdu output stream."""
    
    def __init__(self):
        """Creates a PduOutputStream and initializes it."""
        super(PduOutputStream, self).__init__()
        
    def set_value_at(self, offset, b):
        """Writes a b at offset."""
        _pos = self.position
        self.position = offset
        self.write_byte(b)
        self.position = _pos
        
class PduInputStream(MemoryStream):
    """Implements a MemoryStream dedicated to pdu input stream."""
    
    def __init__(self, bytes_):
        """Creates a PduOutputStream and initializes it with the given bytes_ _buffer."""
        super(PduInputStream, self).__init__(bytes_)

    def _get_remaining_bytes(self):
        return self._length - self._position
    remaining_bytes = property(_get_remaining_bytes, "remaining bytes in stream.")
    
    def expect(self, byte):
        b = self.read_byte()
        if b != byte:
            raise XmlPduException("Illegal value %d in pdu at pos %d. " % ( b, self.position ))
        
    def inspect(self):
        b = self.read_byte()
        if b != -1:
            self._position -= 1
        return b
        
if __name__ == "__main__":
#    stream = MemoryStream()
#    print stream
#    for i in range(30):
#        stream.write_byte(i)
#        print stream
#
#    stream.position = 0
#
#    while True:
#        _b = stream.read_byte()
#        print "_b: %d" % _b
#        if _b == -1:
#            break
#        print stream
    
#    stream = PduOutputStream()
#    for i in range(5):
#        stream.write_byte(i)
#    print stream
#    stream.set_value_at(2, 0)
#    print stream
    
    stream = PduInputStream(range(5))
    print stream
    stream.erase(0, 2)
    print stream
    

    
